import 'dart:async';
import 'dart:convert';
import 'dart:io' show Platform;
import 'package:device_info_plus/device_info_plus.dart';
import 'package:fk_user_agent/fk_user_agent.dart';
import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;

import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:uuid/uuid.dart';

import 'ConscentInitializer.dart';
import 'Constants.dart';
import 'Model/APIMode.dart';
import 'Model/PageViewEvent.dart';
import 'Model/content_access_model.dart';
import 'Model/getUserDetail.dart';
import 'Model/loginChallengeget.dart';
import 'Model/loginChallengepost.dart';
import 'Model/logout.dart';
import 'Model/subscription_access_model.dart';
import 'SharedPref.dart';

class ConscentMethods {
  ContentAccessModel? contentAccessModel;

  String? loginId;
  Future<ContentAccessModel?> getContentAccess() async {
    final prefs = await SharedPreferences.getInstance();
    var loginId = prefs.getString('loginID');
    if (loginId == null) {
      LoginChallengepost loginChallengepost = await createLogInChallenge();

      loginId = loginChallengepost.loginChallengeId;
    }

    print('loginId $loginId');
    var responDtta = await getSessionID(loginId.toString());
    return responDtta;
  }

  Future<LoginChallengepost> createLogInChallenge() async {
    var response = await http
        .post(Uri.parse('${APIMode.BASE_URL}api/v1/login-challenge'), body: {
      "loginChallengeId":
          '${DateTime.now().millisecondsSinceEpoch.toString()}${Uuid().v4()}',
      "redirectTo": ""
    });
    print(response.body);
    if (response.statusCode == 201) {
      var responseString = response.body;
      String loginChallengeId =
          loginChallengepostFromJson(responseString).loginChallengeId;
      final prefs = await SharedPreferences.getInstance();
      print(loginChallengeId);
      await prefs.setString('loginID', loginChallengeId);
      print('dev login challange id');
      print(prefs.getString('loginID'));
      return loginChallengepostFromJson(responseString);
    } else {
      onPluginError(response.body);
      throw Exception('Failed to create album.');
    }
  }

  Future<ContentAccessModel?> getSessionID(String loginId) async {
    var uri = Uri.parse('${APIMode.BASE_URL}api/v1/login-challenge/$loginId');
    print(uri);
    var response = await http.get(uri);
    if (response.statusCode == 200) {
      final prefs = await SharedPreferences.getInstance();
      print('dev sessionnnnnnnnnnnnn id');
      print(prefs.getString('sessionId'));
      LoginChallengeget loginChallengeget =
          loginChallengegetFromJson(response.body);
      print('getSessionID ${loginChallengeget.toJson()}');
      String sessionId = loginChallengeget.sessionId.toString();

      prefs.setString('sessionId', sessionId);

      var responseDta = await submitDatas(
          ConscentInitializer.mClientId.toString(),
          ConscentInitializer.mContentId.toString(),
          sessionId);
      return responseDta;
    } else {
      onPluginError(response.body);
    }
  }

  Future<ContentAccessModel?> submitDatas(
    String clientId,
    String clientContentId,
    String sessionId,
  ) async {
    print('submitDatas sessionId = $sessionId');
    var response = await http
        .post(Uri.parse('${APIMode.BASE_URL}api/v1/content/access'), body: {
      "clientId": clientId,
      "clientContentId": clientContentId,
      "sessionId": sessionId,
    });

    if (response.statusCode == 201) {
      contentAccessModel =
          ContentAccessModel.fromJson(jsonDecode(response.body));
      //  print(dataMains);
      if (contentAccessModel?.sessionExists == null &&
          contentAccessModel?.accessTimeLeft == null) {
        onPluginError(response.body);
      }
      print('contentAccessModel ${contentAccessModel?.toJson()}');

      if (contentAccessModel?.signature != null) {
        if (contentAccessModel?.unlockedWithPass == true) {
          Constants.accessType = "PASS";
        }
        if (contentAccessModel?.unlockedWithSubscription == true) {
          Constants.accessType = "SUB";
        }
        Constants.accessType = "CONTENT";
      }
      bool? sessionExists = false;
      sessionExists = contentAccessModel?.sessionExists;
      Constants.sessionExists = sessionExists!;
      return contentAccessModel;
    } else {
      onPluginError(response.body);
    }
  }

  Future<String?> getSessionIDs() async {
    var prefs = await SharedPreferences.getInstance();
    var isLogin = prefs.getString('loginID');
    return prefs.getString('sessionId');
  }

  Future<String?> isLogin() async {
    var prefs = await SharedPreferences.getInstance();
    var isLogin = prefs.getString('loginID');
    return isLogin;
  }

  // methods for subscription
  Future<Subscription_Access?> geSubscriptionAccess() async {
    final prefs = await SharedPreferences.getInstance();
    loginId = prefs.getString('loginID');
    print(loginId.toString());
    print('logindata');
    var responDtta = await getSessionIDForSubs(loginId.toString());
    // setState(() {});
    return responDtta;
  }

  Future<Subscription_Access?> getSessionIDForSubs(String loginId) async {
    var uri = Uri.parse('${APIMode.BASE_URL}api/v1/login-challenge/$loginId');
    print(uri);
    var response = await http.get(uri);
    if (response.statusCode == 200) {
      final prefs = await SharedPreferences.getInstance();
      LoginChallengeget loginChallengeget =
          loginChallengegetFromJson(response.body);
      print('getSessionID ${loginChallengeget.toJson()}');
      String sessionId = loginChallengeget.sessionId.toString();
      prefs.setString('sessionId', sessionId);
      var responseDta = await getPstssForSubs();
      return responseDta;
    } else {
      onPluginError(response.body);
    }
  }

  Future<Subscription_Access?> getPstssForSubs() async {
    final prefs = await SharedPreferences.getInstance();
    var sessionIds = prefs.getString('sessionId');
    var uri = Uri.parse(
        '${APIMode.BASE_URL}api/v1/subscription/access?sessionId=$sessionIds&clientId=${ConscentInitializer.mClientId}');
    print(uri);
    var response = await http.get(uri);
    if (response.statusCode == 200) {
      var subscription_Access =
          Subscription_Access.fromJson(jsonDecode(response.body));

      print(subscription_Access.signature);
      print('datasub');
      return subscription_Access;
    } else {
      onPluginError(response.body);
    }
  }

  Future<Logout?> userLogOut(String sessionId) async {
    String? sessionId = await SharedPref().getSessionId();
    Logout logout;
    var response = await http.post(
        Uri.parse(
            '${APIMode.BASE_URL}api/v1/auth/logout-via-sessionid?sessionId=$sessionId'),
        body: {
          "sessionId": sessionId,
        });
    print(response.body);
    if (response.statusCode == 201) {
      logout = logoutFromJson(response.body);
      return logout;
    } else {
      onPluginError(response.body);
      logout = logoutFromJson(response.body);
    }

    return logout;
  }

  Future<GetUserDetail?> getUser() async {
    print('getSessionId------');
    GetUserDetail? getUserDetail;

    final prefs = await SharedPreferences.getInstance();
    String? sessionId = await SharedPref().getSessionId();
    sessionId = prefs.getString('sessionId');
    print(sessionId.toString());

    var uri = Uri.parse(
        '${APIMode.BASE_URL}api/v1/user/$sessionId?clientId=${ConscentInitializer.mClientId}');
    print(uri);
    var response = await http.get(uri);

    if (response.statusCode == 200) {
      getUserDetail = GetUserDetail.fromJson(jsonDecode(response.body));
      print('getusermain  $getUserDetail');
      return getUserDetail;
    } else {
      onPluginError(response.body);
    }
    return getUserDetail;
  }

  Future<void> getOSVersion() async {
    await FkUserAgent.init();
    Constants.userAgent = FkUserAgent.webViewUserAgent;
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    if (Platform.isAndroid) {
      AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;

      print('androidInfo version ${androidInfo.version.release}');
      print('androidInfo model ${androidInfo.model}');
      Constants.osVersion = androidInfo.version.release;
      Constants.mobileModel = androidInfo.model;
      Constants.osName = "Android";
    } else if (Platform.isIOS) {
      IosDeviceInfo iosDeviceInfo = await deviceInfo.iosInfo;

      Constants.osVersion = iosDeviceInfo.systemVersion;
      Constants.mobileModel = iosDeviceInfo.model;
      Constants.osName = "Iphone";
    }
  }

  void onTapFun() {
    print('idleTimeSpent ${Constants.idleTimeSpent}');
    if (Constants.idleTimeSpent < 10) {
      Constants.idleTimeSpent = 0;
    } else {
      Constants.totalIdleTimeSpent =
          Constants.totalIdleTimeSpent + Constants.idleTimeSpent;
      Constants.idleTimeSpent = 0;
    }
  }

  int transformMilliSeconds(int milliseconds) {
    int hundreds = (milliseconds / 10).truncate();
    int seconds = (hundreds / 100).truncate();
    int minutes = (seconds / 60).truncate();
    int hours = (minutes / 60).truncate();

    return seconds;
  }

  var messageBodyPageView = PageViewMessageBody();
  static Timer? _timer;
  Future<void> pageViewDisposeEvent(double heights, double onScollDepth) async {
    messageBodyPageView.pageLength = heights.toInt();
    messageBodyPageView.scrollDepth = onScollDepth.toInt();
    int startTime = 0;
    // activeTime = totaltime - idealtime
    int idealTime = 0;
    if (Constants.totalIdleTimeSpent == 0) {
      idealTime = Constants.idleTimeSpent;
      if (idealTime < 10) {
        idealTime = 0;
      }
    } else {
      if (Constants.idleTimeSpent > 10) {
        idealTime = Constants.totalIdleTimeSpent;
      }
    }
    int? totalTimeSpend = Constants.totalTime;
    print('totalTimeSpend $totalTimeSpend  $startTime }');

    int activeTime = totalTimeSpend - idealTime;
    if (activeTime < 0) {
      activeTime *= -1;
    }
    messageBodyPageView.idleTimeSpent = idealTime;
    messageBodyPageView.activeTimeSpent = activeTime;
    messageBodyPageView.totalTimeSpent = totalTimeSpend;
    messageBodyPageView.unlockedStatus = Constants.sessionExists;
    messageBodyPageView.accessType = Constants.accessType;
    Constants.totalTime = 0;
    Constants.idleTimeSpent = 0;
    Constants.totalIdleTimeSpent = 0;

    _timer?.cancel();

    var paywallViewEvent = PageViewEvent(messageBody: [messageBodyPageView]);
    paywallViewEvent.topic = "demoTopic";
    final headers = {'Content-Type': 'application/json'};
    Map<String, dynamic> body = paywallViewEvent.toJson();
    String jsonBody = json.encode(body);
    final encoding = Encoding.getByName('utf-8');
    print("pageViewEvent  $jsonBody");
    var response = await http.post(
      Uri.parse('https://collections.tsbdev.co/collect/event'),
      headers: headers,
      body: jsonBody,
      encoding: encoding,
    );
    int? statusCode = response.statusCode;
    String responseBody = response.body;
    print("statusCode  ${statusCode.toString()}");
    print("responseBody  ${responseBody.toString()}");
  }

  Future<void> pageViewEvent() async {
    print('pageViewEvent start ');
    await ConscentMethods().getOSVersion();

    _timer = Timer.periodic(const Duration(seconds: 1), (timer) {
      Constants.totalTime = Constants.totalTime + 1;
      Constants.idleTimeSpent = Constants.totalTime;
      print('startTime  ${Constants.totalTime}');
    });
    var uuid =
        '${DateTime.now().millisecondsSinceEpoch.toString()}${const Uuid().v4()}';

    var anonId = await SharedPref().getAnonId();
    var userType = await SharedPref().getUserType();
    userType ??= "NEW";

    anonId ??=
        '${DateTime.now().millisecondsSinceEpoch.toString()}${Uuid().v4()}';

    DateFormat dateFormat = DateFormat("yyyy-MM-dd HH:mm:ss");
    String createdAt = dateFormat.format(DateTime.now());
    messageBodyPageView.id = uuid;
    messageBodyPageView.anonId = anonId;
    messageBodyPageView.userType = userType;
    messageBodyPageView.deviceType = 'Mobile';
    messageBodyPageView.browser = "";
    messageBodyPageView.userAgent = Constants.userAgent;
    messageBodyPageView.eventType = "VIEW";
    messageBodyPageView.eventLocation = "PAGE";
    messageBodyPageView.clientId = '${ConscentInitializer.mClientId}';
    messageBodyPageView.osName = Constants.osName;
    messageBodyPageView.userAgent = Constants.userAgent;
    messageBodyPageView.contentId = '${ConscentInitializer.mContentId}';
    messageBodyPageView.createdAt = createdAt;
    messageBodyPageView.updatedAt = createdAt;
    messageBodyPageView.premiumContent = !Constants.freeAccess;
    messageBodyPageView.pageType = "APP";

    messageBodyPageView.osVersion = Constants.osVersion;
    messageBodyPageView.mobileModel = Constants.mobileModel;
    await SharedPref().setUserType("REPEAT");
  }

  void onPluginError(String error) {
    if (kDebugMode) {
      print('onPluginError1: $error');
    }
    debugPrint('onPluginError2: $error');
  }
}
